/*
*       This is the Raiden cipher source code.
*       This file includes the encoding and decoding routines
*
*       Developed by Javier Polimn and Julio Cesar Hernandez
*       Updated by A.Myasnikov to Raiden-2 with 256 bit key (fast edition)
*       Last update: 30th of Jan, 2009.
*
*       Web: www.darksoftware.narod.ru
*/


#include <mem.h>;

void __export __stdcall
crypt (unsigned long *data, unsigned long *result, unsigned long *key)
{

  unsigned long b0 = data[0];
  unsigned long b1 = data[1];
  unsigned long i;
  unsigned long sk;
  unsigned long k[8];
  memcpy (k, key, 32);


  for (i = 0; i != 24; i++)
    {
      sk = k[i % 8] =
	k[i % 8] ^ ((k[4] + k[5]) + ((k[6] + k[7]) ^ (k[4] << k[6])));
      b0 += ((sk + b1) << 9) ^ ((sk - b1) ^ ((sk + b1) >> 14));
      i++;
      sk = k[i % 8] =
	k[i % 8] ^ ((k[0] + k[1]) + ((k[2] + k[3]) ^ (k[0] << k[2])));
      b1 += ((sk + b0) << 9) ^ ((sk - b0) ^ ((sk + b0) >> 14));
    }
  result[0] = b0;
  result[1] = b1;
}




void __export __stdcall
decrypt (unsigned long *data, unsigned long *result, unsigned long *key)
{
  unsigned long b0 = data[0];
  unsigned long b1 = data[1];
  unsigned long i;
  unsigned long k[8];
  unsigned long subkeys[24];
  memcpy (k, key, 32);



  for (i = 0; i != 24; i++)
    {
      subkeys[i] = k[i % 8] = k[i % 8] ^
	((k[4] + k[5]) + ((k[6] + k[7]) ^ (k[4] << k[6])));
      i++;
      subkeys[i] = k[i % 8] = k[i % 8] ^
	((k[0] + k[1]) + ((k[2] + k[3]) ^ (k[0] << k[2])));
    }

  for (i = 23; i != -1; i--)
    {

      b1 -=
	((subkeys[i] + b0) << 9) ^ ((subkeys[i] -
				     b0) ^ ((subkeys[i] + b0) >> 14));
      i--;
      b0 -=
	((subkeys[i] + b1) << 9) ^ ((subkeys[i] -
				     b1) ^ ((subkeys[i] + b1) >> 14));
    }
  result[0] = b0;
  result[1] = b1;

}
